
/**
  * Sikiのスレッドや板を読んだり投稿する際の通信を編集します 
  * フォルダをpluginsへ配置して有効化
  * fsやcrypto等のnodeの標準モジュールを requireすることが出来ます
  * 
  * @version 0.0.4
 */

/**
 * プラグインのタイプ
 * request-hookは通信内容をフックして処理を行います
 */
module.exports.type = 'request-hook'

/**
 * プラグインの情報
 */
module.exports.meta = {
  name: 'request-hook',
  description: 'request-hook',
  version: '1.0.4'
}

/**
 * 設定を同じ場所にあるconfig.jsonから読み取ります
 * @type {Object<string, string|number|boolean>}}
 */
const settings = require('./config')

/**
 * @typedef {Object} Conf - request_hook_conf.jsへデータを保存します 
 * @property {(key: string) => string} get - 保存したデータの読み込み
 * @property {(key: string, value: string) => string} set - 任意のデータを保存
 * @property {(key: string) => string} delete - データの削除
 */

/**
 * @callback SimpleFetch - 通信を行います
 * @param {string} url
 * @param {{ [key: string]: string }} headers - リクエストヘッダ
 * @param {"GET" | "POST" | "HEAD" | "PUT" | "DELETE"} method - HTTP リクエストメソッド
 * @param {string} body - エンコードされたクエリストリング
 * @param {string} encoding - 通信先の文字エンコーディングを必要に応じて指定(指定しない場合自動で推測されます)
 * @return {Promise<{headers:{ [key: string]: string }, body: string}>}
 */

/**
 * @typedef {Object} Log - ログを残します
 * @property {(message: string) => void } error - ログを残す(errorレベル) 
 * @property {(message: string) => void } warn - ログを残す(warnレベル) 
 * @property {(message: string) => void } info - ログを残す(infoレベル) 
 * @property {(message: string) => void } verbose - ログを残す(verboseレベル) 
 * @property {(message: string) => void } debug - ログを残す(debugレベル) 
 */

/**
 * URLリクエスト前のヘッダやパラメータを変更します。
 * 
 * @param {Object} tools - データの保存などで利用できるツール
 * @param {Conf} tools.conf
 * @param {SimpleFetch} tools.simplefetch
 * @param {Log} tools.log
 * @param {{ [key: string]: string }} tools.siki_values - workspaceidやuser_dir,log_dir等の値が入っています
 * @param {string} url - リクエストされるURL
 * @param {"GET" | "POST" | "HEAD" | "PUT" | "DELETE"} method - HTTP リクエストメソッド
 * @param {{ [key: string]: string }} headers - リクエストされるhttpヘッダー 全て小文字となります
 * @param {string} payload - 本来リクエストされるクエリストリング aa=bb&cc=dd&ee= ... という形式でパーセントエンコード済み
 * @param {{ [key: string]: string }} query - リクエストされるクエリー エンコードはされていません
 * @return {{url?: string, headers?: { [key: string]: string }, payload?: string, method?: string}} - URLのリクエストパラメータ 返さない場合は本来の内容で処理が進みます
 * 
 * @example - 特定のサイトへのアクセスの場合にUser Agentを変更しクエリストリングを変更
 * const beforeRequest = async (tools, url, method, headers, payload, query) => {
 *   if(url.match(/example\.com/)) {
 *     headers['user-agent'] = 'NCSA_Mosaic/2.0 (Windows 3.1)'
 *     payload = 'a=1&b=%E3%81%82'
 *   }
 *   return { headers, payload }
 * }
 */
module.exports.beforeRequest = async (tools, url, method, headers, payload, query) => {
  const { conf, simplefetch, log, siki_values } = tools
  // 処理を記述

  return { url, headers, payload, method }
}

/**
 * 
 * 
 * @module afterRequest URLリクエスト後のヘッダや内容を変更します。
 * @param {Object} tools - データの保存などで利用できるツール
 * @param {Conf} tools.conf
 * @param {SimpleFetch} tools.simplefetch
 * @param {Log} tools.log
 * @param {{ [key: string]: string }} tools.siki_values - workspaceidやuser_dir,log_dir等の値が入っています
 * @param {string} url - リクエストされたURL
 * @param {number} statusCode - HTTPステータスコード
 * @param {{ [key: string]: string }} response_headers - サーバーから帰って来たhttpヘッダー 全て小文字となります
 * @param {string} response_body - 受け取ったレスポンス内容 レスポンスのエンコーディングに関係なくユニコードに変換済み
 * @return {{statusCode?: number, response_headers?: { [key: string]: string }, response_body?: string}} - レスポンスヘッダと内容を返してください 返さない場合は本来の内容で処理が進みます
 * 
 * @example - 受け取った内容からストレージへ保存
 * const afterRequest = async (tools, url, response_headers, response_body) => {
 *   const { conf } = tools
 *   if(url.match(/example\.com/)) {
 *     conf.set('x', response_body)
 *   }
 *   return { response_headers, response_body }
 * }
 */
module.exports.afterRequest = async (tools, url, statusCode, response_headers, response_body) => {
  const { conf, simplefetch, log, siki_values } = tools
  // 処理を記述

  return { statusCode, response_headers, response_body }
}
